﻿#pragma once
#include "LLGL/LLGL.h"
#include "misc/mirage_type.h"
#include <spdlog/spdlog.h>
#include <filesystem>

#define MIRAGE_VERSION_MAJOR 0
#define MIRAGE_VERSION_MINOR 1
#define MIRAGE_VERSION_PATCH 0

namespace mirage {
    renderer_api get_default_renderer_api();
    struct init_info : window_descriptor, swap_chain_descriptor {
        renderer_api api = get_default_renderer_api();
    };

    int run(const init_info& in_init_info);

    const duration_type& get_delta_time();
    duration_type get_total_time();

    LLGL::RenderSystem* get_renderer();
    LLGL::CommandQueue* get_main_command_queue();
    uint64_t get_frame_index();

    std::filesystem::path get_shader_path();
    void set_shader_path(const std::filesystem::path& in_shader_path);

    template<typename T>
    void llgl_deleter(T* ptr) {
        if (auto renderer = get_renderer()) {
            renderer->Release(*ptr);
        }
    }
}
