#pragma once
#include "LLGL/LLGL.h"
#include "misc/color.h"
#include "misc/enum_class_flags.h"
#include <Eigen/Eigen>
#include <cstdint>

namespace mirage {
    using transform_type = Eigen::Affine2d;
    using color_type = LLGL::ColorRGBAub;
    using linear_color_type = linear_color;
    using point_type = Eigen::Vector2f;

    struct vertex_shader_param {
        union {
            float x, a;
        };
        union {
            float y, b;
        };
        union {
            float z, c;
        };
        union {
            float w, d;
        };

        template<typename T>
        vertex_shader_param(Eigen::Matrix<T, 4, 1> in_vec) : x(in_vec.x()), y(in_vec.y()), z(in_vec.z()), w(in_vec.w()) {}
        vertex_shader_param() : x(0), y(0), z(0), w(0) {}

        float operator[](const std::size_t index) const {
            switch (index) {
            case 0:
                return x;
            case 1:
                return y;
            case 2:
                return z;
            case 3:
                return w;
            default:
                return {};
            }
        }
    };

    struct vertex {
        point_type position;
        point_type uv;
        linear_color_type color;
        vertex_shader_param param[3];
    };
    struct triangle {
        uint32_t index[3];
    };

    struct rect {
        Eigen::Vector2f pos;
        Eigen::Vector2f size;

        [[nodiscard]] Eigen::Vector2f get_center() const {
            return pos + size * 0.5f;
        }
        [[nodiscard]] Eigen::Vector2f get_top_left() const {
            return pos;
        }
        [[nodiscard]] Eigen::Vector2f get_top_right() const {
            return {pos.x() + size.x(), pos.y()};
        }
        [[nodiscard]] Eigen::Vector2f get_bottom_left() const {
            return {pos.x(), pos.y() + size.y()};
        }
        [[nodiscard]] Eigen::Vector2f get_bottom_right() const {
            return pos + size;
        }

        [[nodiscard]] bool contains_point(const Eigen::Vector2f& in_point) const {
            return in_point.x() >= pos.x() && in_point.x() <= pos.x() + size.x() && in_point.y() >= pos.y() && in_point.y() <= pos.y() + size.y();
        }
    };
}
