#include "window.h"

#include "mirage.h"

#if MIRAGE_PLATFORM_WINDOWS
#include "LLGL/Platform/Win32/Win32NativeHandle.h"
#elif MIRAGE_PLATFORM_LINUX
#include "LLGL/Platform/Linux/LinuxNativeHandle.h"
#elif MIRAGE_PLATFORM_ANDROID
#include "LLGL/Platform/Android/AndroidNativeHandle.h"
#endif

namespace mirage {
    window::window(const LLGL::WindowDescriptor& in_desc) {
        surface = LLGL::Window::Create(in_desc);
        if (surface == nullptr) {
            throw std::runtime_error("无法创建窗口");
        }
        init_event_listener();
    }

    window::~window() {
        delete context;
        surface.reset();
    }

    bool window::create_context(const swap_chain_descriptor& in_desc) {
        context = new render_context();
        if (context->init(in_desc, surface))
            return true;
        delete context;
        context = nullptr;
        return false;
    }

#if !MIRAGE_PLATFORM_IOS && !MIRAGE_PLATFORM_MACOS
    void* window::get_native_handle() const {
        LLGL::NativeHandle native_handle{};
        if (surface->GetNativeHandle(&native_handle, sizeof(native_handle))) {
            return native_handle.window;
        }
        return nullptr;
    }
#endif

    void window::render_update(const duration_type& in_delta_time) {
        if (context) {
            context->render_update(in_delta_time);
        }
    }
}
