#pragma once
#include "misc/mirage_type.h"
#include "renderer/render_context.h"
#include <LLGL/LLGL.h>

namespace mirage {
    class window {
        friend class desktop_window_event_listener;
        friend class mobile_window_event_listener;
    public:
        explicit window(const LLGL::WindowDescriptor& in_desc);
        virtual ~window();

        bool create_context(const swap_chain_descriptor& in_desc);
        [[nodiscard]] auto get_render_context() const {
            return context;
        }
        [[nodiscard]] void* get_native_handle() const;

        void render_update(const duration_type& in_delta_time);
        void set_title(const std::u8string& in_title);
        [[nodiscard]] std::u8string get_title() const;
        void set_fullscreen(bool in_fullscreen);
        [[nodiscard]] bool is_fullscreen() const;
        void set_resizable(bool in_resizable);
        [[nodiscard]] bool is_resizable() const;
        void set_borderless(bool borderless);
        [[nodiscard]] bool is_borderless() const;
        void set_centered(bool in_centered);

        void show(bool in_show);
        void move(const LLGL::Offset2D& position);
        void move_delta(const LLGL::Offset2D& delta);
        void resize(const LLGL::Extent2D& size);
        [[nodiscard]] LLGL::Extent2D get_size(bool in_client_size = true) const;

    protected:
        virtual void on_quit(bool& out_veto) {}
        virtual void on_key_down(LLGL::Key in_key_code) {}
        virtual void on_key_up(LLGL::Key in_key_code) {}
        virtual void on_double_click(LLGL::Key in_key_code) {}
        virtual void on_char(wchar_t in_chr) {}
        virtual void on_wheel_motion(int in_motion) {}
        virtual void on_local_motion(const LLGL::Offset2D& in_position) {}
        virtual void on_global_motion(const LLGL::Offset2D& in_motion) {}
        virtual void on_resize(const LLGL::Extent2D& in_client_area_size);
        virtual void on_update() {}
        virtual void on_get_focus() {}
        virtual void on_lost_focus() {}
        virtual void on_tap_gesture(const LLGL::Offset2D& in_position, std::uint32_t in_num_touches) {}
        virtual void on_pan_gesture(const LLGL::Offset2D& in_position, std::uint32_t in_num_touches, float in_dx, float in_dy, LLGL::EventAction in_action) {}
    private:
        void init_event_listener();
        std::shared_ptr<LLGL::Surface> surface;
        render_context* context = nullptr;
    };
} // namespace mirage
