#pragma once
#include <vector>
#include <memory>
#include "window.h"
#include <shared_mutex>

namespace mirage {
    class window_manager {
    public:
        void update(const duration_type& in_delta_time);

        bool init(const init_info& in_info);
        bool destroy();

        [[nodiscard]] bool should_exit() const {
            return should_exit_flag;
        }

        std::weak_ptr<window> create_window(const LLGL::WindowDescriptor& desc);
        void destroy_window(const window& in_window);
        std::weak_ptr<window> get_main_window();
        [[nodiscard]] auto get_windows() const {
            std::vector<std::weak_ptr<window>> result;
            for (const auto& w : windows) {
                result.push_back(w);
            }
            return result;
        }
    private:
        std::vector<std::shared_ptr<window>> windows;
        std::vector<std::shared_ptr<window>> to_destroy;

        void render_thread_func();
        time_type last_time               = {};
        std::atomic_bool should_exit_flag = false;
    };
} // namespace mirage
